/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：C++ 信息安全性设计准则
 * 项目描述：用于检查C++源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.cxx.rules.checkers;

import com.sonar.cxx.sslr.api.AstNode;
import com.sonar.cxx.sslr.api.Grammar;
import org.sonar.check.Priority;
import org.sonar.check.Rule;
import org.sonar.cxx.parser.CxxGrammarImpl;
import org.sonar.cxx.squidbridge.annotations.ActivatedByDefault;
import org.sonar.cxx.squidbridge.annotations.SqaleConstantRemediation;
import org.sonar.cxx.squidbridge.checks.SquidCheck;

import static com.sonar.cxx.sslr.api.GenericTokenType.IDENTIFIER;

/**
 * C++规则检查器的实现示例
 *
 * @author GuoXin
 * @date 2024/1/6
 */
@Rule(key = "ABCVarNameChecker", name = "变量名称不可以是ABC", description = "变量名称不可以是ABC", priority = Priority.INFO, tags = {"28suo"})
@ActivatedByDefault
@SqaleConstantRemediation("5min")
public class ABCVarNameChecker extends SquidCheck<Grammar> {

    /**
     * 初始化规则检查器
     */
    @Override
    public void init() {
        // 订阅要检查AST节点类型，用于在visitNode方法中检查该类型节点
        this.subscribeTo(
                CxxGrammarImpl.declaratorId
        );
    }

    /**
     * 检查AST节点
     *
     * @param astNode 要处理的AST节点，该节点类型为通过subscribeTo方法订阅的类型
     */
    @Override
    public void visitNode(AstNode astNode) {
        var idNode = astNode.getLastChild(IDENTIFIER);
        if (idNode != null) {
            String identifier = idNode.getTokenValue();
            if ("ABC".equals(identifier)) {
                getContext().createLineViolation(this, "变量名称不可以是ABC", idNode);
            }
        }
    }
}
