/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：Java 信息安全性设计准则
 * 项目描述：用于检查Java源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.java.rules.checkers;

import org.sonar.check.Rule;
import org.sonar.java.model.expression.BinaryExpressionTreeImpl;
import org.sonar.plugins.java.api.IssuableSubscriptionVisitor;
import org.sonar.plugins.java.api.tree.*;

import java.util.List;


@Rule(key = "UpperCycleLimitRuleChecker")
public class UpperCycleLimitRuleChecker extends IssuableSubscriptionVisitor {
    @Override
    public List<Tree.Kind> nodesToVisit() {
        return List.of(Tree.Kind.METHOD);
    }

    @Override
    public void visitNode(Tree tree) {
        //将Tree强转成MethodTree
        MethodTree methodTree = (MethodTree) tree;
        List<VariableTree> args = methodTree.parameters();
        BlockTree blockTree = methodTree.block();
        blockTree.accept(new MethodBlockVisitor(this, args));
    }


    class MethodBlockVisitor extends BaseTreeVisitor {
        protected UpperCycleLimitRuleChecker checker;
        final List<VariableTree> args;

        public MethodBlockVisitor(UpperCycleLimitRuleChecker checker, List<VariableTree> args) {
            this.args = args;
            this.checker = checker;
        }

        @Override
        public void visitForStatement(ForStatementTree fnode) {
            var ffnode = fnode.condition();
            if (ffnode instanceof BinaryExpressionTreeImpl) {
                ExpressionTree leftOperand = ((BinaryExpressionTreeImpl) ffnode).leftOperand();
                checkVar(leftOperand);
                ExpressionTree rightOperand = ((BinaryExpressionTreeImpl) ffnode).rightOperand();
                checkVar(rightOperand);
            }
        }

        @Override
        public void visitWhileStatement(WhileStatementTree wnode) {
            var wwnode = wnode.condition();
            ExpressionTree teea = ((BinaryExpressionTreeImpl) wwnode).leftOperand();
            checkVar(teea);
        }

        @Override
        public void visitDoWhileStatement(DoWhileStatementTree dnode) {
            var ddnode = dnode.condition();
            ExpressionTree teeas = ((BinaryExpressionTreeImpl) ddnode).leftOperand();
            checkVar(teeas);
        }


        private void checkVar(ExpressionTree operand) {
            if (operand instanceof IdentifierTree) {
                IdentifierTree identifierTree = (IdentifierTree) operand;
                var name = identifierTree.name();
                for (VariableTree varTree : args) {
                    if (varTree.simpleName().name().equals(name)) {
                        System.out.println(identifierTree.firstToken().range().start().line() + ": " + name + " " + "规定循环次数的上限，在将用户输入的数据用于循环条件前进行验证用户输入的数据是否超过上限");
                        checker.context.reportIssue(checker, identifierTree, "规定循环次数的上限，在将用户输入的数据用于循环条件前进行验证用户输入的数据是否超过上限");
                    }
                }
            }
        }


    }
}
